"""The wavfrmstream commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - WAVFRMStream?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Wavfrmstream(SCPICmdRead):
    """The ``WAVFRMStream`` command.

    **Description:**
        - This query only command returns WFMQUTPRE? and CURVESTREAM? data for the waveforms
          specified by the DATASOURCE command. This command is similar to sending both WFMOUTPRE?
          and CURVESTREAM?, with the additional provision that each CURVESTREAM response to WAVFRMS?
          has a WFMOUTPRE response prepended to it. This helps guarantee a continuous synchronized
          preamble and curve.

    **Usage:**
        - Using the ``.query()`` method will send the ``WAVFRMStream?`` query.
        - Using the ``.verify(value)`` method will send the ``WAVFRMStream?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WAVFRMStream?
    """

    def __init__(
        self, device: Optional["PIDevice"] = None, cmd_syntax: str = "WAVFRMStream"
    ) -> None:
        super().__init__(device, cmd_syntax)



"""SMU Model 2600 device driver module."""
import string

from abc import ABC
from functools import cached_property
from typing import Tuple, Union

from tm_devices.commands import (
    SMU2601BCommands,
    SMU2601BPulseCommands,
    SMU2602BCommands,
    SMU2604BCommands,
    SMU2606BCommands,
    SMU2611BCommands,
    SMU2612BCommands,
    SMU2614BCommands,
    SMU2634BCommands,
    SMU2635BCommands,
    SMU2636BCommands,
    SMU2651ACommands,
    SMU2657ACommands,
)
from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.pi.source_measure_units.source_measure_unit import SourceMeasureUnit


@family_base_class
class SMU2600(SourceMeasureUnit, ABC):
    """Base SMU2600 device driver."""

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return tuple(string.ascii_lowercase[: self.total_channels])

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        # Grab the total channel count based on whether the last digit in the model is even/odd
        return 2 if (not int([x for x in self.model if x.isdigit()][-1]) % 2) else 1

    @property
    def commands(
        self,
    ) -> Union[
        SMU2601BCommands,
        SMU2602BCommands,
        SMU2604BCommands,
        SMU2606BCommands,
        SMU2611BCommands,
        SMU2612BCommands,
        SMU2614BCommands,
        SMU2634BCommands,
        SMU2635BCommands,
        SMU2636BCommands,
        SMU2601BPulseCommands,
        SMU2651ACommands,
        SMU2657ACommands,
    ]:
        """Return the device commands."""
        return self._commands  # pragma: no cover

    ################################################################################################
    # Public Methods
    ################################################################################################
    def get_eventlog_status(self) -> Tuple[bool, str]:
        """Help function for getting the eventlog status.

        Returns:
            Boolean indicating no error, String containing concatenated contents of event log.
        """
        result_allev = False
        allev_result_str = '0,"No events to report - queue empty"'

        # instrument returns exponential numbers so converting to float before int
        # TODO: switch to auto-generated command once SMU2601B-PULSE is ready
        if not (err_count := int(float(self.query("print(errorqueue.count)")))):
            result_allev = True
        else:
            allev_result_list = [self.query("print(errorqueue.next())") for _ in range(err_count)]
            allev_result_str = ",".join(allev_result_list)
        return result_allev, allev_result_str

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _reboot(self) -> None:
        """Perform the actual rebooting code."""
        # TODO: implement
        raise NotImplementedError(
            f"``.reboot()`` is not yet implemented for the {self.__class__.__name__} driver"
        )



"""The autoset commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - AUTOSet {EXECute|UNDo|VFields|VIDeo|VLines}
    - AUTOSet:OVErlay {ON | OFF}
    - AUTOSet:PERcent <NR3>
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class AutosetPercent(SCPICmdWrite):
    """The ``AUTOSet:PERcent`` command.

    **Description:**
        - Sets or queries the percent of the full vertical scale that the signal should cover after
          Autoset is run when Overlay is turned on.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``AUTOSet:PERcent value`` command.

    **SCPI Syntax:**

    ::

        - AUTOSet:PERcent <NR3>
    """


class AutosetOverlay(SCPICmdWrite):
    """The ``AUTOSet:OVErlay`` command.

    **Description:**
        - Sets or queries the autoset overlay feature. If Autoset Overlay is turned on, Autoset will
          place all signals center vertically on-screen with the vertical settings on each channel
          configured so that the amplitude of the signal occupies n divisions = Percent/10
          divisions.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``AUTOSet:OVErlay value`` command.

    **SCPI Syntax:**

    ::

        - AUTOSet:OVErlay {ON | OFF}
    """


class Autoset(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet`` command.

    **Description:**
        - This command (no query format) sets the vertical, horizontal, and trigger controls of the
          instrument to automatically acquire and display the selected waveform. (To autoset a video
          waveform, the video trigger must be set to video standard, not custom. Video arguments
          require video hardware.) This is equivalent to pressing the front panel AUTOSET button.
          For a detailed description of autoset functionality, see Autoset in the index of the
          online help for your instrument.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``AUTOSet value`` command.

    **SCPI Syntax:**

    ::

        - AUTOSet {EXECute|UNDo|VFields|VIDeo|VLines}

    **Info:**
        - ``EXECute`` runs the autoset routine; this is equivalent to pressing the front panel
          AUTOSET button. If the display is set to a PAL, MV, or IRE graticule, this argument forces
          the graticule display to full mode (frame, grid, and cross hair).
        - ``UNDo`` returns the instrument to the setting prior to executing an autoset.
        - ``VFields`` autosets the displayed waveform.
        - ``VIDeo`` autosets the displayed waveform.
        - ``VLines`` autosets the displayed waveform.

    Properties:
        - ``.overlay``: The ``AUTOSet:OVErlay`` command.
        - ``.percent``: The ``AUTOSet:PERcent`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "AUTOSet") -> None:
        super().__init__(device, cmd_syntax)
        self._overlay = AutosetOverlay(device, f"{self._cmd_syntax}:OVErlay")
        self._percent = AutosetPercent(device, f"{self._cmd_syntax}:PERcent")

    @property
    def overlay(self) -> AutosetOverlay:
        """Return the ``AUTOSet:OVErlay`` command.

        **Description:**
            - Sets or queries the autoset overlay feature. If Autoset Overlay is turned on, Autoset
              will place all signals center vertically on-screen with the vertical settings on each
              channel configured so that the amplitude of the signal occupies n divisions =
              Percent/10 divisions.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``AUTOSet:OVErlay value`` command.

        **SCPI Syntax:**

        ::

            - AUTOSet:OVErlay {ON | OFF}
        """
        return self._overlay

    @property
    def percent(self) -> AutosetPercent:
        """Return the ``AUTOSet:PERcent`` command.

        **Description:**
            - Sets or queries the percent of the full vertical scale that the signal should cover
              after Autoset is run when Overlay is turned on.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``AUTOSet:PERcent value`` command.

        **SCPI Syntax:**

        ::

            - AUTOSet:PERcent <NR3>
        """
        return self._percent
